$ontext

   OLS plus statistics

   Erwin Kalvelagen, november 2000

$offtext

set i /i1*i40/;

$include expdata.inc

set j /'constant','coeff1'/;
alias (j,jj);
alias (j,k);
alias (i,ii);

parameters
   X(i,j)   'the X matrix in standard OLS notation (dependent variables)'
   y(i)     'the y vector in standard OLS notation (independent variables)'
   XX(j,jj) "the matrix (X'X)"
   Xy(j)    "the vector (X'y)"
;

X(i,'constant') = 1;
X(i,'coeff1')   = data(i,'income');

y(i) = data(i,'expenditure');

variables
    residual(i)  'error term'
    sseA          'sum of squared errors'
;
variables
    b(j)       'parameters to estimate'
;

equations
    fit(i)     'the linear model'
    obj        'objective'

;

obj..     sseA =e= sum(i, sqr(residual(i)));
fit(i)..  data(i,'expenditure') =e= sum(j, b(j)*X(i,j)) + residual(i);


model ols1 /obj,fit/;

solve ols1 minimizing sseA using nlp;

display b.l, sseA.l;

parameters
   yhat(i)       'predicted y'
   A(i,i)        "Theil's A matrix: I - (1/n) (iota*iota')"
;


scalars
   sse
   sst           'total sum of squares'
   ssr           'regression sum of squares'
   n             'number of observations'
   df            'degrees of freedom'
   r2            'r-square'
   r2adj         'r-square adjusted'
   llf           'log of the likelihood function'
   sigma_squared 'variance of the estimate'
   sigma         'standard error of the estimate'
   pi            '3.1415...'
;

yhat(i) = sum(j, x(i,j)*b.l(j));
*residual(i) = y(i) - yhat(i);

sse = sum(i, sqr(residual.l(i)));

n = card(i);
df = n - 2;
sigma_squared = sse/df;
sigma = sqrt(sigma_squared);
pi = 4*arctan(1);

A(i,ii) = 1$sameas(i,ii) - (1/n);

sst = sum((i,ii), data(i,'expenditure')*A(i,ii)*data(ii,'expenditure'));
ssr = sst - sse;

r2 = 1-sse/sst;

r2adj = 1 - ((n-1)/df)*(1-r2);

llf = -(n/2)*log(2*pi*sse/n)-(n/2);



display "--------------statistics---------------------",
        sse,
        sigma_squared,
        sigma,
        r2,
        r2adj,
        llf;


alias (j,jjj);

equations
    dummy_eq   'dummy objective equation'
;

variables
    dummy_var  'dummy objective variable'
;

XX(j,jj) = sum(i, X(i,j)*X(i,jj));
dummy_eq..  dummy_var =e= b('constant');

variable var(j,jj) 'variance of the estimators';
equation variance(j,jj);
variance(j,jj).. sum(jjj, XX(j,jjj)*var(jjj,jj)) =e= sigma_squared$sameas(j,jj);

*option lp=ls;
model mvar  /dummy_eq,variance/;
solve mvar using lp minimizing dummy_var;

parameters
    se(j)      'standard error'
    t(j)       "t ratio's"
    partial(j)
;

se(j) = sqrt(var.l(j,j));

t(j) =  b.l(j)/se(j);

partial(j) = t(j)/sqrt(sqr(t(j))+df);

display "--------------------------------------------------",
        se,
        t,
        partial
;


set prob /p1,p2,p3,p4,p5,p6/;
parameter probval(prob) /
p1 0.10,  p2 0.05,  p3 0.025,  p4 0.01,  p5 0.005,  p6 0.001
/;
parameter qnorm(prob) /
p1 1.281552, p2 1.644854, p3 1.959964, p4 2.326348, p5 2.575829, p6 3.090232
/;

set ival confidence interval /'lo', 'up'/;
scalar ndf degrees of freedom;
ndf = card(i) - card(j);
scalar alpha significance level /0.025/;
scalar qt critical value;
abort$(ndf<30) "Normal approximation not valid";
qt = sum(prob$(probval(prob)=0.025), qnorm(prob));
parameter ols_conf_ival(j,ival);
ols_conf_ival(j,'lo') = b.l(j) - qt*se(j);
*ols_conf_ival(j,lo) = 10 ;
ols_conf_ival(j,'up') = b.l(j) + qt*se(j);
*ols_conf_ival(j,up) = 12;
display b.l, ols_conf_ival;


